#ifndef __XMLIO_CField__
#define __XMLIO_CField__

/// xmlioserver headers ///
#include "xmlioserver_spl.hpp"
#include "group_factory.hpp"

#include "functor.hpp"
#include "functor_type.hpp"

#include "declare_group.hpp"

namespace xmlioserver
{
   namespace tree
   {
      /// ////////////////////// Déclarations ////////////////////// ///

      class CFieldGroup;
      class CFieldAttributes;
      class CField;

      class CFile;
      class CGrid;

      ///--------------------------------------------------------------

      // Declare/Define CFieldAttribute
      BEGIN_DECLARE_ATTRIBUTE_MAP(CField)
#include "field_attribute.conf"
      END_DECLARE_ATTRIBUTE_MAP(CField)

      ///--------------------------------------------------------------
      class CField
         : public CObjectTemplate<CField>
         , public CFieldAttributes
      {
            /// friend ///
            friend class CFile;

            /// typedef ///
            typedef CObjectTemplate<CField>   SuperClass;
            typedef CFieldAttributes SuperClassAttribute;

         public :

            typedef CFieldAttributes RelAttributes;
            typedef CFieldGroup      RelGroup;

            /// Constructeurs ///
            CField(void);
            explicit CField(const StdString & id);
            CField(const CField & field);       // Not implemented yet.
            CField(const CField * const field); // Not implemented yet.

            /// Accesseurs ///
            boost::shared_ptr<CField> getDirectFieldReference(void) const;
            const boost::shared_ptr<CField> getBaseFieldReference(void) const;
            const std::vector<boost::shared_ptr<CField> > & getAllReference(void) const;

            boost::shared_ptr<CGrid> getRelGrid(void) const ;
            boost::shared_ptr<CFile> getRelFile(void) const ;

            const StdString & getBaseFieldId(void) const;

            /// Mutateur ///
            void setRelFile(const boost::shared_ptr<CFile> _file);

         public :

            /// Test ///
            bool hasDirectFieldReference(void) const;

            /// Traitements ///
            void solveRefInheritance(void);
            void solveGridReference(void);
            void solveOperation(void);

            /// Destructeur ///
            virtual ~CField(void);

            /// Accesseurs statiques ///
            static StdString GetName(void);
            static StdString GetDefName(void);

         private :

            /// Propriétés privées ///
            boost::shared_ptr<CField> baseRefObject;
            std::vector<boost::shared_ptr<CField> > refObject;

            boost::shared_ptr<CGrid> grid ;
            boost::shared_ptr<CFile> file;

            func::CFunctor * foperation;

      }; // class CField

      ///--------------------------------------------------------------

      // Declare/Define CFieldGroup and CFieldDefinition
      DECLARE_GROUP(CField);

   } // namespace tree

   ///-----------------------------------------------------------------

   template <>
      void CGroupTemplate<CField, CFieldGroup, CFieldAttributes>::solveRefInheritance(void);

} // namespace xmlioserver

#endif // __XMLIO_CField__
